# Copyright (C) 2006-2007 Red Hat, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

from gettext import gettext as _

import os
import logging
from ConfigParser import ConfigParser

import gobject
import dbus

from sugar import env
from sugar.util import unique_id
from sugar.graphics.xocolor import XoColor

from jarabe.model.buddy import BuddyModel
from jarabe.model import neighborhood


_model = None


class FriendBuddyModel(BuddyModel):
    __gtype_name__ = 'SugarFriendBuddyModel'

    _NOT_PRESENT_COLOR = '#D5D5D5,#FFFFFF'

    def __init__(self, nick, key, account=None, contact_id=None, groups=[]):
        self._online_buddy = None

        BuddyModel.__init__(self, nick=nick, key=key, account=account,
                            contact_id=contact_id, groups=groups)

        neighborhood_model = neighborhood.get_model()
        neighborhood_model.connect('buddy-added', self.__buddy_added_cb)
        neighborhood_model.connect('buddy-removed', self.__buddy_removed_cb)

        buddy = neighborhood_model.get_buddy_by_key(key)
        if buddy is not None:
            self._set_online_buddy(buddy)

    def __buddy_added_cb(self, model_, buddy):
        if buddy.key != self.key:
            return
        self._set_online_buddy(buddy)

    def _set_online_buddy(self, buddy):
        self._online_buddy = buddy
        self._online_buddy.connect('notify::color', self.__notify_color_cb)
        self.notify('color')
        self.notify('present')

        if buddy.nick != self.nick:
            self.nick = buddy.nick
        if buddy.contact_id != self.contact_id:
            self.contact_id = buddy.contact_id
        if buddy.account != self.account:
            self.account = buddy.account

    def __buddy_removed_cb(self, model_, buddy):
        if buddy.key != self.key:
            return
        self._online_buddy = None
        self.notify('color')
        self.notify('present')

    def __notify_color_cb(self, buddy, pspec):
        self.notify('color')

    def is_present(self):
        return self._online_buddy is not None

    present = gobject.property(type=bool, default=False, getter=is_present)

    def get_color(self):
        if self._online_buddy is not None:
            return self._online_buddy.color
        else:
            return XoColor(FriendBuddyModel._NOT_PRESENT_COLOR)

    color = gobject.property(type=object, getter=get_color)

    def get_handle(self):
        if self._online_buddy is not None:
            return self._online_buddy.handle
        else:
            return None

    handle = gobject.property(type=object, getter=get_handle)


class Friends(gobject.GObject):
    __gsignals__ = {
        'friend-added': (gobject.SIGNAL_RUN_FIRST, gobject.TYPE_NONE,
                         ([object])),
        'friend-removed': (gobject.SIGNAL_RUN_FIRST, gobject.TYPE_NONE,
                           ([str])),
    }

    def __init__(self):
        gobject.GObject.__init__(self)
        self._path = os.path.join(env.get_profile_path(), 'friends')
        self._groups_path = os.path.join(env.get_profile_path(), 'groups')
        self.reinit()

    def reinit(self):
        self._friends = {}
        self._groups = {}
        self.load()

    def has_buddy(self, buddy):
        return self.check_buddy_existence_by_key(buddy.get_key())

    """
    Ideally, this should be the only publically exposed API, since
    only the buddy_key is the deciding factor of the existence of
    a friend. There can never be two friends of the same key.
    """
    def check_buddy_existence_by_key(self, buddy_key):
        return buddy_key in self._friends

    def add_friend(self, buddy_info):
        self._friends[buddy_info.get_key()] = buddy_info
        self.emit('friend-added', buddy_info)

    def add_friend_to_group(self, buddy_key, group_name,
                            save_group=True):
        if not self.__group_exists(group_name):
            return _('Cannot add to group !! Create this group first !!')

        if self.__friend_exists_in_group(group_name, buddy_key):
            return _('Friend already exists in this group')

        # If we reach here, it is safe to add this buddy :)
        self._groups[group_name]['friends'][buddy_key] = {}

        # Also, update the association from the buddy side, and save.
        groups = self._get_groups_of_a_friend(buddy_key)
        if group_name not in groups:
            groups.append(group_name)
            groups.sort()
            self.save()

        if save_group:
            self.save_groups()

    def remove_friend_group_assoc_using_friend_key(self, group_name,
            friend_key, save_friend=True, save_group=True):
        if not self.__group_exists(group_name):
            return _('No such group exists !!')

        # Remove association from friend.
        if group_name in self._friends[friend_key].get_groups():
            self._friends[friend_key].get_groups().remove(group_name)

        # Remove association from group.
        if friend_key in self._groups[group_name]['friends'].keys():
            del self._groups[group_name]['friends'][friend_key]

        if save_friend:
            self.save()
        if save_group:
            self.save_groups()

    def _get_groups_of_a_friend(self, friend_key):
        if friend_key not in self._friends.keys():
            return []

        return self._friends[friend_key].get_groups()

    def add_group(self, group_name, save_group=True):
        if self.__group_exists(group_name):
            return _('A group with the same name already exists !!'
                     'Choose a different group-name.')

        # If we reach here, it is safe to add a group of this name.
        self._groups[group_name] = {}
        self._groups[group_name]['friends'] = {}
        self._groups[group_name]['last-operation'] = 'NONE.'
        if save_group:
            self.save_groups()

    def remove_group(self, group_name, save_friend=True,
            save_group=True):
        if not self.__group_exists(group_name):
            return _('No group exists of this name !!')

        # Remove this group, from all the friend-associations.
        for friend_key in self._friends.keys():
            friend_groups_list = self._friends[friend_key].get_groups()
            if group_name in friend_groups_list:
                friend_groups_list.remove(group_name)
        if save_friend:
            self.save()

        # Remove the group itself.
        del self._groups[group_name]
        if save_group:
            self.save_groups()

    def __group_exists(self, group_name):
        if group_name in self._groups.keys():
            return True
        return False

    def __friend_exists_in_group(self, group_name, friend_key):
        if friend_key in \
                self._groups[group_name]['friends'].keys():
            return True
        return False

    """
    This method is useful in the case, when the buddy is to be added as
    a friend in deferred sense. For example, the buddies list may first be
    fetched by virtue of their being online, but may go offline by the
    time they are about to be saved. So, in that case, we retrieve the
    parameters that are required for saving in the early stages itself.
    """
    def make_friend_by_parameters(self, buddy_key, buddy_nick, buddy_account, buddy_contact_id):
        if not self.check_buddy_existence_by_key(buddy_key):
            buddy = FriendBuddyModel(key=buddy_key,
                                     nick=buddy_nick,
                                     account=buddy_account,
                                     contact_id=buddy_contact_id)
            self.add_friend(buddy)
            self.save()

    def make_friend(self, buddy):
        self.make_friend_by_parameters(buddy.key, buddy.nick,
                                       buddy.account, buddy.contact_id)

    def remove_groups_of_friend(self, buddy_info, save_friend=True,
            save_group=True):
        groups = self._get_groups_of_a_friend(buddy_info.get_key())
        for group in groups:
            self.remove_friend_group_assoc_using_friend_key(group,
                    buddy_info.get_key(), False, False)

        if save_friend:
            self.save()
        if save_group:
            self.save_groups()

    def remove(self, buddy_info):
        # First remove its association from all its groups
        self.remove_groups_of_friend(buddy_info, False, True)

        # Now, remove the friend-entity itself.
        del self._friends[buddy_info.get_key()]
        self.save()

        self.emit('friend-removed', buddy_info.get_key())

    def __iter__(self):
        return self._friends.values().__iter__()

    def _get_friend_by_key(self, key):
        if not (key in self._friends.keys()):
            return _('No friend found !!')
        return self._friends[key]

    def _get_friend_keys_of_group(self, group_name):
        if not self.__group_exists(group_name):
            return []

        return self._groups[group_name]['friends'].keys()

    def _get_groups(self):
        # Sascha's wonderful feedback: always export only the thing
        # required. Here, we required only the group-names; so export
        # just the keys
        return self._groups.keys()

    def _set_groups(self, groups):
        self._groups = groups

    def _get_group_by_key_name(self, group_name):
        if not self.__group_exists(group_name):
            return None
        return self._groups[group_name]

    def _get_last_group_operation(self, group_name):
        if not self.__group_exists(group_name):
            return _('No group of this name exists !!')
        return self._groups[group_name]['last-operation']

    def _set_last_group_operation(self, group_name, operation):
        if not self.__group_exists(group_name):
            return _('No group exists.')
        self._groups[group_name]['last-operation'] = operation
        self.save_groups

    def _get_last_operation_status_of_friend_in_group(self, group_name,
                                                      friend_key):
        if not self.__group_exists(group_name):
            return _('No group exists.')

        if not self.__friend_exists_in_group(group_name, friend_key):
            return _('Friend does not exist in group')

        if 'last-operation-status' in \
                self._groups[group_name]['friends'][friend_key].keys():
            return self._groups[group_name]['friends'][friend_key]['last-operation-status']

    def _set_last_operation_status_of_friend_in_group(self, group_name,
                                                      friend_key, status,
                                                      save_group=True):
        if not self.__group_exists(group_name):
            return _('No group exists.')

        if not self.__friend_exists_in_group(group_name, friend_key):
            return _('Friend does not exist in group')

        self._groups[group_name]['friends'][friend_key]['last-operation-status'] = \
                status

        if save_group:
            self.save_groups()

    def _set_last_operation_status_of_friends_in_group_with_common_status(
            self, group_name, status):
        if not self.__group_exists(group_name):
            return _('No group exists.')

        friend_keys = self._get_friend_keys_of_group(group_name)
        for friend_key in friend_keys:
            self._set_last_operation_status_of_friend_in_group(group_name,
                                                               friend_key,
                                                               status,
                                                               False)

        # Save the groups in one go.
        self.save_groups()

    def load(self):
        cp = ConfigParser()

        try:
            success = cp.read([self._path])
            if success:
                for key in cp.sections():
                    # HACK: don't screw up on old friends files
                    if len(key) < 20:
                        continue

                    # Check for the existence of 'groups' option (for
                    # backwards compatability)
                    groups = []
                    if cp.has_option(key, 'groups'):
                        groups = eval(cp.get(key, 'groups'))

                    buddy = FriendBuddyModel(key=key, nick=cp.get(key,
                        'nick'), groups=groups)
                    self.add_friend(buddy)
        except Exception:
            logging.exception('Error parsing friends file')

        self.__load_groups()

    def __load_groups(self):
        cp = ConfigParser()

        try:
            success = cp.read([self._groups_path])
            if success:
                for group_name in cp.sections():
                    self._groups[group_name] = {}
                    self._groups[group_name]['friends'] = \
                            eval(cp.get(group_name, 'friends'))
                    self._groups[group_name]['last-operation'] = \
                            cp.get(group_name, 'last-operation')
        except:
            logging.exception('Error while loading config')

    def save(self):
        cp = ConfigParser()

        for friend in self:
            section = friend.get_key()
            cp.add_section(section)
            cp.set(section, 'nick', friend.get_nick())
            cp.set(section, 'groups', friend.get_groups())

        fileobject = open(self._path, 'w')
        cp.write(fileobject)
        fileobject.close()

    def save_groups(self):
        cp = ConfigParser()

        for group in self._groups.keys():
            cp.add_section(group)
            cp.set(group, 'friends', self._groups[group]['friends'])
            cp.set(group, 'last-operation',
                    self._groups[group]['last-operation'])

        fileobject = open(self._groups_path, 'w')
        cp.write(fileobject)
        fileobject.close()

def get_model():
    global _model
    if _model is None:
        _model = Friends()
    return _model
