# Copyright (C) 2012 Simon Schampijer <erikos@sugarlabs.org>
# Copyright (C) 2012 Ajay Garg        <ajay@activitycentral.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA

import gtk
import gobject
from gettext import gettext as _

from sugar.graphics import style
from sugar.graphics.alert import Alert
from sugar.graphics.icon import Icon

from jarabe.model import buddy
from jarabe.model import friends
from jarabe.model import neighborhood

from jarabe.controlpanel.sectionview import SectionView
from jarabe.controlpanel.inlinealert import InlineAlert

owner_model = buddy.get_owner_instance()
friends_model = friends.get_model()
neighborhood_model = neighborhood.get_model()

class LeftAlignedLabelWidget(gtk.HBox):

    def __init__(self, label):
        gtk.HBox.__init__(self)
        label_widget = gtk.Label(label)
        label_widget.set_line_wrap(True)
        self.pack_start(label_widget, expand=False)
        self.show_all()


class CenterAlignedLabelWidget(gtk.VBox):

    def __init__(self, label):
        gtk.VBox.__init__(self)
        label_widget = gtk.Label(label)
        label_widget.set_line_wrap(True)
        self.pack_start(label_widget, expand=False)
        self.show_all()


class BuddyWidget(gtk.HBox):

    def __init__(self, buddy):
        gtk.HBox.__init__(self)
        self._buddy = buddy

        self._check_button = gtk.CheckButton(label=self._buddy.nick)
        self._check_button.set_active(False)
        self.pack_start(self._check_button, expand=False)
        self.show_all()

    def _is_buddy_selected(self):
        return self._check_button.get_active()

    def _get_buddy_key(self):
        return self._buddy.key

    def _get_buddy_nick(self):
        return self._buddy.nick

    def _get_buddy_account(self):
        if hasattr(self._buddy, 'account'):
            return self._buddy.account
        return None

    def _get_buddy_contact_id(self):
        if hasattr(self._buddy, 'contact_id'):
            return self._buddy.contact_id
        return None


class AddRemoveWidget(gtk.VBox):

    def __init__(self, label, group_detail, add_button_clicked_cb,
                 remove_button_clicked_cb, index):
        gtk.VBox.__init__(self)
        self.set_homogeneous(False)
        self.set_spacing(10)

        self._potential_new_group = False
        self._group_name = label
        self._group_detail = group_detail

        self._primary_box = gtk.HBox()
        self._primary_box.set_homogeneous(False)
        self._primary_box.set_spacing(10)
        self.pack_start(self._primary_box, expand=False)
        self._primary_box.show_all()

        self._index = index
        self._add_button_added = False
        self._remove_button_added = False

        self._label = gtk.Entry()
        self._label.set_text(label)

        # Do not allow an already existing group name to be modified.
        if len(label) > 0:
            self._label.set_sensitive(False)
        # Else, this is a potentially new group.
        # Mark this is as new.
        # However, this will ACTUALLY be a new group,
        # only if it is given a name.
        # That check will be done, after the user clicks the final save
        # button.
        else:
            self._potential_new_group = True

        self._primary_box.pack_start(self._label, expand=False)

        if not self._potential_new_group:
            self._details_button = gtk.Button(_('View Details'))
            self._view_id = self._details_button.connect('clicked', self.__view_details_cb)
            self._primary_box.pack_start(self._details_button,
                    expand=False)

        add_icon = Icon(icon_name='list-add')
        self._add_button = gtk.Button()
        self._add_button.set_image(add_icon)
        self._add_button.connect('clicked',
                                 add_button_clicked_cb,
                                 self)

        remove_icon = Icon(icon_name='list-remove')
        self._remove_button = gtk.Button()
        self._remove_button.set_image(remove_icon)
        self._remove_button.connect('clicked',
                                    remove_button_clicked_cb,
                                    self)

        self.__add_add_button()
        self.__add_remove_button()

        self._details_table = gtk.VBox()
        self._details_table.set_spacing(20)
        self._details_table.show_all()
        self.pack_start(self._details_table, expand=False)

        if self._potential_new_group:
            info_label = LeftAlignedLabelWidget(_('You may batch add'
                ' from the following available online buddies. Any'
                ' currently offline buddy may be added later from the'
                ' neighborhood view, when it comes online.'))
            self.pack_start(info_label, expand=False)
            self._friends_list_box = gtk.VBox()
            self._friends_list_box.set_homogeneous(True)
            self._friends_list_box.set_spacing(20)
            self.pack_start(self._friends_list_box, expand=False)
            self._populate_friends_list()

        self.pack_start(gtk.HSeparator())

        self._primary_box.show_all()
        self.show_all()

    def _populate_friends_list(self):
        buddies = neighborhood_model.get_buddies()
        for buddy in buddies:

            # Only make the buddy visible, if it has a valid key at
            # this time.
            if ((hasattr(buddy, 'key')) and (buddy.key is not None)):

                # Do not show self :)
                if buddy.key == owner_model.get_key():
                    continue

                self._friends_list_box.pack_start(BuddyWidget(buddy))

        self._friends_list_box.show_all()

    def _get_buddy_widgets(self):
        return self._friends_list_box.get_children()

    def _get_index(self):
        return self._index

    def _set_index(self, value):
        self._index = value

    def _get_entry(self):
        return self._label.get_text()

    def __add_add_button(self):
        self._primary_box.pack_start(self._add_button, expand=False)
        self._add_button_added = True

    def _remove_remove_button_if_not_already(self):
        if self._remove_button_added:
            self.__remove_remove_button()

    def __remove_remove_button(self):
        self._primary_box.remove(self._remove_button)
        self._remove_button_added = False

    def _add_remove_button_if_not_already(self):
        if not self._remove_button_added:
            self.__add_remove_button()

    def __add_remove_button(self):
        self._primary_box.pack_start(self._remove_button, expand=False)
        self._remove_button_added = True

    def __activate_view_id(self):
        self._details_button.disconnect(self._hide_id)
        self._view_id = self._details_button.connect('clicked',
                self.__view_details_cb)

    def __activate_hide_id(self):
        self._details_button.disconnect(self._view_id)
        self._hide_id = self._details_button.connect('clicked',
                self.__hide_details_cb)

    def __view_details_cb(self, widget):
        if self._group_detail is None:
            return

        last_operation_value = friends_model._get_last_group_operation(self._group_name)

        self._last_operation_box = gtk.VBox()
        self._last_operation_box.pack_start(LeftAlignedLabelWidget(_('Last'
            ' Operation On This Group :: ')), expand=False)
        self._last_operation_box.pack_start(CenterAlignedLabelWidget(
            last_operation_value), expand=False)
        self._last_operation_box.show_all()

        self._details_table.pack_start(self._last_operation_box,
                expand=False)

        self._container = gtk.Table()
        self._details_table.pack_start(self._container, expand=False)

        headings_list = [_('Nick'), _('Last Operation Status')]
        for i in range(0, len(headings_list)):
            self._container.attach(gtk.Label(headings_list[i]), i, i+1,
                    0, 1)
        for i in range(0, len(headings_list)):
            self._container.attach(gtk.Label(''), i, i+1, 1, 2)

        index = 2
        friend_keys_of_group = \
                friends_model._get_friend_keys_of_group(self._group_name)

        for friend_key in friend_keys_of_group:
            friend_model = friends_model._get_friend_by_key(friend_key)
            label_widgets = []
            nick = friend_model.get_nick()
            label_widgets.append(CenterAlignedLabelWidget(nick))

            last_operation_status = \
                    friends_model._get_last_operation_status_of_friend_in_group(self._group_name,
                                                                                friend_key)
            label_widgets.append(CenterAlignedLabelWidget(last_operation_status))

            for i in range(0, len(label_widgets)):
                self._container.attach(label_widgets[i], i, i+1, index,
                        index+1)
            index = index + 1

        self._container.show_all()

        self._details_button.set_label(_('Hide Details'))
        self.__activate_hide_id()

    def __hide_details_cb(self, widget):
        self._details_table.remove(self._last_operation_box)
        self._details_table.remove(self._container)

        self._details_button.set_label(_('View Details'))
        self.__activate_view_id()


class MultiWidget(gtk.VBox):

    def __init__(self):
        gtk.VBox.__init__(self)
        self.set_spacing(10)

    def _add_widget(self, label, metadata):
        new_widget = AddRemoveWidget(label,
                                     metadata,
                                     self.__add_button_clicked_cb,
                                     self.__remove_button_clicked_cb,
                                     len(self.get_children()))
        self.add(new_widget)
        self.show_all()
        self._update_remove_button_statuses()

    def _add_blank_entry(self):
        self._add_widget('', None)

    def __add_button_clicked_cb(self, add_button,
                                      add_button_container):
        self._add_blank_entry()
        self._update_remove_button_statuses()

    def __remove_button_clicked_cb(self, remove_button,
                                   remove_button_container):
        # Remove group from the model.
        group_name = remove_button_container._get_entry()
        friends_model.remove_group(group_name)

        # Remove group from the view.
        self.remove(remove_button_container)

        self._update_remove_button_statuses()

    def _update_remove_button_statuses(self):
        children = self.get_children()

        # Now, if there is only one entry, remove-button
        # should not be shown.
        if len(children) == 1:
            children[0]._remove_remove_button_if_not_already()

        # Alternatively, if there are more than 1 entries,
        # remove-button should be shown for all.
        if len(children) > 1:
            for child in children:
                child._add_remove_button_if_not_already()

    def set_groups(self, groups):
        self._groups = groups

    def _pre_save_operations(self):
        for child in self.get_children():
            if child._potential_new_group:
                group_name = child._get_entry()
                if len(group_name) > 0:
                    friends_model.add_group(group_name, False)

                    # Also, add all the selected buddies as friends.
                    buddy_widgets = child._get_buddy_widgets()
                    for widget in buddy_widgets:
                        if widget._is_buddy_selected():
                            # Add as friend.
                            friends_model.make_friend_by_parameters(
                                    widget._get_buddy_key(),
                                    widget._get_buddy_nick(),
                                    widget._get_buddy_account(),
                                    widget._get_buddy_contact_id())

                            # Add as friend in group.
                            friends_model.add_friend_to_group(widget._get_buddy_key(),
                                                              group_name,
                                                              False)

        # Perform just one disk-write for groups.
        friends_model.save_groups()


class Configuration(SectionView):
    def __init__(self, model, alerts=None):
        SectionView.__init__(self)

        self._model = model

        self.set_border_width(style.DEFAULT_SPACING * 2)
        self.set_spacing(style.DEFAULT_SPACING)
        group = gtk.SizeGroup(gtk.SIZE_GROUP_HORIZONTAL)

        workspace = gtk.VBox()
        workspace.show()

        separator = gtk.HSeparator()
        workspace.pack_start(separator, expand=False)

        label_friend_groups = gtk.Label(_('Groups configuration'))
        label_friend_groups.set_alignment(0, 0)
        workspace.pack_start(label_friend_groups, expand=False)

        box_friend_groups = gtk.VBox()
        box_friend_groups.set_border_width(style.DEFAULT_SPACING * 2)
        box_friend_groups.set_spacing(style.DEFAULT_SPACING)

        self._widget_table = MultiWidget()
        box_friend_groups.pack_start(self._widget_table, expand=False)

        save_button = gtk.Button()
        save_button.set_alignment(0, 0)
        save_button.set_label('Save')
        save_button.connect('clicked', self.__save_button_clicked_cb)
        box_save_button = gtk.HBox()
        box_save_button.set_homogeneous(False)
        box_save_button.pack_start(save_button, expand=False)
        box_save_button.show_all()

        box_friend_groups.pack_start(box_save_button, expand=False)

        box_friend_groups.show_all()
        workspace.pack_start(box_friend_groups, expand=False)

        scrolled = gtk.ScrolledWindow()
        scrolled.set_policy(gtk.POLICY_NEVER, gtk.POLICY_AUTOMATIC)
        scrolled.add_with_viewport(workspace)
        scrolled.show()
        self.add(scrolled)

        workspace.show_all()
        self.setup()

    def setup(self):
        groups = friends_model._get_groups()
        groups.sort()

        if len(groups) == 0:
            self._widget_table._add_blank_entry()
        else:
            for group_name in groups:
                group_detail = \
                        friends_model._get_group_by_key_name(group_name)
                self._widget_table._add_widget(group_name, group_detail)

    def __save_button_clicked_cb(self, save_button):
        save_button.set_sensitive(False)
        self._widget_table._pre_save_operations()
